@@ -1,6 +1,8 @@
 context("gls objects")
 set.seed(20190513)
 
+skip_if_not_installed("nlme")
+
 library(nlme, quietly=TRUE, warn.conflicts=FALSE)
 
 data(Ovary, package = "nlme")
@@ -79,20 +81,34 @@ test_that("CR2 and CR4 are target-unbiased", {
   expect_true(check_CR(lm_AR1_power, vcov = "CR4"))
 })
 
-test_that("getData works.", {
+test_that("get_data works.", {
   re_order <- sample(nrow(Ovary))
   egg_scramble <- Ovary[re_order,]
   gls_scramble <- gls(follicles ~ sin(2*pi*Time) + cos(2*pi*Time), 
                       data = egg_scramble)
-  scramble_dat <- getData(gls_scramble)
+  scramble_dat <- get_data(gls_scramble)
   expect_equal(egg_scramble, scramble_dat)
+  
+  cars_30 <- rownames(mtcars |> subset(mpg < 30))
+  m1 <- gls(mpg ~ hp, data = mtcars |> subset(mpg < 30))
+  vcr1 <- vcovCR(m1, cluster = cars_30, type = "CR0")
+  
+  dat30 <- mtcars |> subset(mpg < 30)
+  m2 <- gls(mpg ~ hp, data = dat30)
+  vcr2 <- vcovCR(m2, cluster = cars_30, type = "CR0")
+  
+  m3 <- gls(mpg ~ hp, data = mtcars |> subset(mpg < 30))
+  m3$data <- dat30
+  vcr3 <- vcovCR(m3, cluster = cars_30, type = "CR0")
+  
+  expect_equal(vcr1, vcr2)
+  expect_equal(vcr2, vcr3)
 })
 
 
 CR_types <- paste0("CR",0:4)
 
 test_that("Order doesn't matter.", {
-  
   check_sort_order(lm_AR1_power, dat = Ovary, 
                    tol = 10^-4, tol2 = 10^-3, tol3 = 10^-3)
 
